var Chatbox = Class.extend({
	init: function(params, element, tree, FunctionRegistrator) {
		this.element = element;
		this.tree = tree.nodes;
		this.FunctionRegistrator = FunctionRegistrator;
		this.currentNode = null;
		this.log = [];
		var defaults = {
			parent: "window",
			title: "Chatbox",
			user: "User",
			hidden: false,
			offset: 0,
			width: 300,
			firstNode: "start"
		}
		
		this.params = $.extend(defaults, params);
		
		if (!(this.params.firstNode in this.tree)) {
			this.params.firstNode = 0;
		}
		
		if (!(typeof this.params.heads === 'undefined')) {
			this.params.title = "<img style='height:100px;' src='" + this.params.heads + "'><image>";
		}
		
		console.log("Chatbox initiate", tree);
		this.render();
	},
	
	messageSent: function(id, msg, delay) {
		console.log("Chatbox messageSent");
		this.boxManager.addMsg(id, msg, delay);
	},

	boxManager: {
		init: function(elem) {
			this.elem = elem;
		},
		addMsg: function(peer, msg, delay) {
			var self = this;
			var box = self.elem.uiChatboxLog;
			var e = document.createElement('div');
			box.append(e);
			$(e).hide();

			var msgElement;
			
			if (peer == "system") {
				msgElement = $("<font color='blue'></font>");
				msgElement.append($("<i></i>").text(msg));
			} else {
				msgElement = document.createElement("span");
				$(msgElement).text(msg);
				var peerName = document.createElement("b");
				$(peerName).text(peer + ": ");
				e.appendChild(peerName);
			}
			
			$(e).append(msgElement);
			$(e).addClass("ui-chatbox-msg");
			$(e).css("maxWidth", $(box).width());
			setTimeout(function() { $(e).fadeIn(); self._scrollToBottom(); }, delay);
		},
		toggleBox: function() {
			this.elem.uiChatbox.toggle();
		},
		_scrollToBottom: function() {
			var box = this.elem.uiChatboxLog;
			box.scrollTop(box.get(0).scrollHeight);
		}
	},

	toggleContent: function(event) {
		this.uiChatboxLog.toggle();
		this.uiChatboxInput.toggle();
		this.uiChatboxSpacer.toggle();
	},

	render: function() {
		var self = this;
		var element = self.element;
		var params = self.params;
		var title = params.title || "No Title";
		var uiChatbox = (self.uiChatbox = element)
			.addClass(
				'ui-widget ' +
				'ui-corner-top ' +
				'ui-chatbox ' +
				'flex-container'
			)
			.attr('outline', 0);

		var uiChatboxSpacer = (self.uiChatboxSpacer = $('<div></div>'))
			.addClass(
				'ui-chatbox-spacer'
			)
			.appendTo(uiChatbox).toggle();

		var uiChatboxTitlebar = (self.uiChatboxTitlebar = $('<div></div>'))
			.addClass(
				'ui-widget-header ' +
				'ui-corner-top ' +
				'ui-chatbox-titlebar ' +
				'ui-dialog-header ' +
				'flex-item'
			)
			.click(function(event) {
				self.toggleContent(event);
			})
			.appendTo(uiChatbox);
		var uiChatboxTitle = (self.uiChatboxTitle = $('<span></span>'))
			.html(title)
			.appendTo(uiChatboxTitlebar);
		var uiChatboxTitlebarMinimize = (self.uiChatboxTitlebarMinimize = $('<a href="#"></a>'))
			.addClass(
				'ui-corner-all ' +
				'ui-chatbox-icon'
			)
			.attr('role', 'button')
			.hover(
				function() { uiChatboxTitlebarMinimize.addClass('ui-state-hover'); },
				function() { uiChatboxTitlebarMinimize.removeClass('ui-state-hover'); }
			)
			.click(function(event) {
				self.toggleContent(event);
				return false;
			})
			.appendTo(uiChatboxTitlebar);
		var uiChatboxTitlebarMinimizeText = $('<span></span>')
			.addClass(
				'ui-icon ' +
				'ui-icon-minusthick'
			)
			.text('minimize')
			.appendTo(uiChatboxTitlebarMinimize);
		var uiChatboxLog = (self.uiChatboxLog = $('<div></div>'))
			.html(self.params.dialogueBefore)
			.addClass(
				'ui-widget-content ' +
				'ui-chatbox-log'
			)
			.appendTo(uiChatbox);
		/*var uiChatboxLogType = (self.uiChatboxLogType = $('<img src="images/type.gif"></img>'))
			.css("position", "absolute")
			.css("bottom", "0")
			.appendTo(uiChatboxLog);*/
		var uiChatboxInput = (self.uiChatboxInput = $('<div></div>'))
			.addClass(
				'ui-widget-content ' +
				'ui-chatbox-input'
			)
			.appendTo(uiChatbox);
		switch (params.parent) {
			case "window":
				self._setWidth(params.width);
				self._position(params.offset);
				break;
			
			case "custom":
				self._setWidth(params.width);
				self._position(params.offset);
				self._setHeight(params.height);
				break;
		}
		
		uiChatboxTitlebar.find('*').add(uiChatboxTitlebar).disableSelection();
		
		self.boxManager.init(self);

		if (!params.hidden) {
			uiChatbox.show();
		}
		self.changeQuestion(self.params.firstNode);
	},

	_setWidth: function(width) {
		this.uiChatboxTitlebar.width(width + "px");
		this.uiChatboxLog.width(width + "px");
		this.uiChatboxInput.css("maxWidth", width + "px");
	},
	_position: function(offset) {
		this.uiChatbox.css("left", offset);
	},
	_setHeight: function(height) {
		this.uiChatbox.css("height", height + "px");
	},

	changeQuestion: function(node) {
		this.log.push(node);
		console.log("Chatbox changeQuestion", node);
		this.currentNode = node;
		node = this.tree[node];
		switch(node.type) {
		case "start":
			this.changeQuestion(node.children[0]);
			break;
		case "delay":
			this.changeQuestion(node.children[0]);
			break;
		case "reference":
			this.changeQuestion(node.reference);
			break;
		case "user":
			this.messageSent(this.params.user, node.text, 0);
			var ok = true;
			if ((typeof node.functions) != "undefined") {
				for (var i = 0;i < node.functions.length;i++) {
					if(this.FunctionRegistrator.exec(node.functions[i].name, node.functions[i].args) == false) {
						ok = false;
					}
				}
			}
			if (ok && node.children.length > 0)
				this.changeQuestion(node.children[0]);
			break;
		case "agents":
			
			var delay = 1500;
			for (var i = 0;i < node.texts.length;i++) {
				if (node.texts[i].agent == "system") {
					this.messageSent(node.texts[i].agent, node.texts[i].text, 0);
					continue;
				}
				this.messageSent(node.texts[i].agent, node.texts[i].text, delay);
				if (i != node.texts.length - 1)
					delay += 700 + node.texts[i + 1].text.length * 20;
				delay += 500;
			}
			
			this.uiChatboxInput.html("");
			
			
			
			if ((typeof node.delay) != "undefined") {
				var delayNode = this.tree[node.delay];
				var _this = this;
				var timeoutID = setTimeout(function(n) { _this.changeQuestion(n); }, delayNode.delay * 1000, node.delay);
				this.timeoutID = timeoutID;
			} else {
				this.timeoutID = undefined;
			}
			
			for (var i = 0;i < node.children.length;i++) {
				if ((typeof node.delay) != "undefined" && node.delay == node.children[i]) {
					continue;
				}
				
				$("<button>", {
					role: "button", 
					text: this.tree[node.children[i]].text
				})
				.addClass("btn")
				.css("width", "100%")
				.prop("disabled", true)
				.bind("click", {
					box: this,
					id: node.children[i],
					timeoutID: (typeof timeoutID === "undefined") ? "undefined" : timeoutID
				}, function(event) {
					event.preventDefault();
					var box = event.data.box;
					var id = event.data.id;
					var timeoutID = event.data.timeoutID;
					
					if (timeoutID != "undefined")
						clearTimeout(timeoutID);
					
					box.changeQuestion(id);
				})
				.appendTo(this.uiChatboxInput);
			}
			var buttons = this.uiChatboxInput.children();
			setTimeout(function() {buttons.prop("disabled", false);}, delay);
		}
	},
	
	reload: function() {
		console.log("Chatbox reload");
		if (!(typeof this.timeoutID === "undefined"))
			clearTimeout(this.timeoutID);
		this.uiChatboxLog.html("");
		this.changeQuestion(0);
	},

	destroy: function() {
		console.log("Chatbox destroy");
		if (!(typeof this.timeoutID === "undefined")) {
			clearTimeout(this.timeoutID);
		}
		this.uiChatbox.html("");
		this.uiChatbox.removeClass("ui-widget");
		this.uiChatbox.removeClass("ui-corner-top");
		this.uiChatbox.removeClass("ui-chatbox");
		this.uiChatbox.removeClass("flex-container");
		this.uiChatbox.removeAttr("outline");
		this.uiChatbox.css("left", "");

		if (this.params.parent == "custom")
			this.uiChatbox.css("height", "");
	},
	
	pause: function() {
		console.log("Chatbox pause");
		
		if (!(typeof this.timeoutID === "undefined"))
			clearTimeout(this.timeoutID);
		
		this.hide();
	},
	
	hide: function() {
		console.log("Chatbox hide");
		this.element.hide();
	},

	show: function() {
		console.log("Chatbox show");
		this.element.show();
	}
});

